==> tag-input-js-0.0.3.tgz: package/src/index.js <==
import { queryElement, createElement, addEvent, hasText, removeElement, removeEvent, parseDOMString } from './utils';

const defaultOptions = {
  cssClass: {
    tagPanel: 'tagi__panel',
    tagElement: 'tagi__element',
    inputElement: 'tagi__input'
  },
  renderRemoveButton() {
    return parseDOMString(
      `<svg width="16" height="16" viewBox="0 0 16 16" fill="none" xmlns="http://www.w3.org/2000/svg">
      <path d="M13.5 2.5L2.5 13.5" stroke="#454647" stroke-miterlimit="10" stroke-linecap="round" stroke-linejoin="round"/>
      <path d="M2.5 2.5L13.5 13.5" stroke="#454647" stroke-miterlimit="10" stroke-linecap="round" stroke-linejoin="round"/>
      </svg>
      `,
      'image/svg+xml'
    );
  },
  setFocus: false,
  initialTags: [],
  onTagChange() {}
};

export default class TagPanel {
  constructor(element, options = {}) {
    this.options = Object.assign({}, defaultOptions, options);
    this.options.cssClass = Object.assign({}, defaultOptions.cssClass, options.cssClass);
    this.panel = this._createPanel(element, this.options.cssClass.tagPanel);
    this.input = this._createInput(this.panel, { className: this.options.cssClass.inputElement, tabIndex: 0 });

    this._subscribeEvents = this._subscribeEvents.bind(this);
    this._unsubscribeEvents = this._unsubscribeEvents.bind(this);
    this._handleTagInput = this._handleTagInput.bind(this);
    this._handleTagBlur = this._handleTagBlur.bind(this);
    this._handleTagClick = this._handleTagClick.bind(this);
    this._handleTagDelete = this._handleTagDelete.bind(this);
    this._handleGlobalKeyup = this._handleGlobalKeyup.bind(this);
    this._notifyCallbackFn = this._notifyCallbackFn.bind(this);
    this._initTags = this._initTags.bind(this);
    this.addTag = this.addTag.bind(this);
    this.destroy = this.destroy.bind(this);
    this.getTagList = this.getTagList.bind(this);
    this.removeTag = this.removeTag.bind(this);

    this._subscribeEvents(this);
    this._initTags();
    if (this.options.setFocus) {
      this.input.focus();
    }
  }

  destroy() {
    this._unsubscribeEvents();
    removeElement(this.panel);
  }

  getTagList() {
    const result = [];
    if (this.panel.hasChildNodes()) {
      const nodes = this.panel.childNodes;
      for (let i = 0; i < nodes.length; i++) {
        const tag = this._getTag(nodes[i], this.options.cssClass.tagElement);
        if (tag) {
          result.push(tag);
        }
      }
    }
    return result;
  }

  addTag(text) {
    this.panel.insertBefore(this._createTagElement(text), this.input);
    setTimeout(() => this._notifyCallbackFn());
  }

  removeTag(tagElement) {
    if (tagElement instanceof HTMLElement) {
      removeElement(tagElement);
      setTimeout(() => this._notifyCallbackFn());
    } else if (typeof tagElement === 'string') {
      const result = [];
      this.panel.childNodes.forEach((el) => {
        if (hasText(el, tagElement)) {
          result.push(el);
        }
      });
      result.forEach((el) => removeElement(el));
      result = null;
      setTimeout(() => this._notifyCallbackFn());
    }
  }

  _initTags() {
    this.options.initialTags.forEach((tag) => this.addTag(tag));
  }

  _notifyCallbackFn() {
    this.options.onTagChange(this.getTagList());
  }

  _createTagElement(tag) {
    const tagEl = createElement('DIV', {
      className: this.options.cssClass.tagElement,
      tabIndex: this.panel.childNodes.length
    });
    const tagTextEl = createElement('SPAN', { innerHTML: tag });
    const tagIconEl = this.options.renderRemoveButton();

    addEvent('click', tagEl, this._handleTagClick);
    addEvent('click', tagIconEl, this._handleTagDelete);
    tagEl.appendChild(tagTextEl);
    tagEl.appendChild(tagIconEl);

    return tagEl;
  }

  _getTag(node, cssClass) {
    let text = '';
    let i = 0;
    if (node.nodeType === 1 && node.nodeName === 'DIV' && node.className === cssClass && node.hasChildNodes()) {
      while (node.childNodes[i]) {
        if (node.childNodes[i].nodeType === 1 && node.childNodes[i].nodeName === 'SPAN') {
          text = (node.childNodes[i].innerText || '').trim();
          i = -1;
        } else {
          i++;
        }
      }
    }
    return text;
  }

  _createPanel(element, className) {
    const panel = createElement('DIV', { className });
    queryElement(element).appendChild(panel);
    return panel;
  }

  _createInput(panel, props) {
    const input = createElement('INPUT', props);
    input.setAttribute('type', 'text');
    panel.appendChild(input);
    return input;
  }

  _subscribeEvents() {
    addEvent('keyup', this.input, this._handleTagInput);
    addEvent('blur', this.input, this._handleTagBlur);
    addEvent('keyup', document, this._handleGlobalKeyup);
  }

  _unsubscribeEvents() {
    removeEvent('keyup', this.input, this._handleTagInput);
    removeEvent('blur', this.input, this._handleTagBlur);
    removeEvent('keyup', document, this._handleGlobalKeyup);
  }

  _handleTagBlur() {
    this.beingUpdate = null;
  }

  _handleTagClick(e) {
    e.stopPropagation();
    this.beingUpdate = e.target;
    this.input.value = this.beingUpdate.innerText;
    this.input.focus();
  }

  _handleTagDelete(e) {
    e.stopPropagation();
    this.beingUpdate = null;
    this.removeTag(e.target.parentNode);
  }

  _handleTagInput(e) {
    e.stopPropagation();
    const code = e.keyCode ? e.keyCode : e.which;
    if (code === 13 && e.target.value.length) {
      if (this.beingUpdate) {
        this.beingUpdate.innerText = e.target.value;
        this.beingUpdate = null;
      } else {
        this.addTag(e.target.value.trim());
      }
      window.setTimeout(
        function() {
          this.target.value = '';
        }.bind(e)
      );
      return;
    } else if (code === 8 && !e.target.value.length) {
      const tagEl = this._getTheLastTag(this.panel, this.options.cssClass.tagElement);
      if (tagEl) {
        if (this.beingUpdate) {
          this.removeTag(this.beingUpdate.parentNode);
          return;
        }
        tagEl.focus();
        return;
      }
    } else if (code === 27) {
      this.beingUpdate = null;
      e.target.value = '';
    }

    if (this.beingUpdate) {
      this.beingUpdate.innerText = e.target.value;
    }
  }

  _handleGlobalKeyup(e) {
    let code = e.keyCode ? e.keyCode : e.which;
    if (
      code === 8 &&
      document.activeElement.parentNode === this.panel &&
      document.activeElement.nodeType === 1 &&
      document.activeElement.nodeName === 'DIV' &&
      document.activeElement.className === this.options.cssClass.tagElement
    ) {
      const focusedEl = document.activeElement;
      if (document.activeElement.previousSibling && document.activeElement.previousSibling.parentNode === this.panel) {
        document.activeElement.previousSibling.focus();
      }
      this.removeTag(focusedEl);
    } else if (code === 27) {
      this.beingUpdate = null;
      this.input.value = '';
    }
  }

  _getTheLastTag(panel, cssClass) {
    let i = panel.childNodes.length - 1 || -1;
    let node = null;
    while (((node = panel.childNodes[i]), node)) {
      if (node.nodeType === 1 && node.nodeName === 'DIV' && node.className === cssClass) {
        return panel.childNodes[i];
      } else {
        i--;
      }
    }
    return null;
  }
}

==> tag-input-js-0.0.4.tgz: package/src/index.js <==
import { queryElement, createElement, addEvent, hasText, removeElement, removeEvent, parseDOMString } from './utils';

const defaultOptions = {
  classnames: {
    tagPanel: 'tagi__panel',
    tagElement: 'tagi__element',
    inputElement: 'tagi__input',
    charsCountElement: 'tagi__chars-panel'
  },
  createRemoveButton(tagInstance) {
    return parseDOMString(
      `<svg width="16" height="16" viewBox="0 0 16 16" fill="none" xmlns="http://www.w3.org/2000/svg">
      <path d="M13.5 2.5L2.5 13.5" stroke="#454647" stroke-miterlimit="10" stroke-linecap="round" stroke-linejoin="round"/>
      <path d="M2.5 2.5L13.5 13.5" stroke="#454647" stroke-miterlimit="10" stroke-linecap="round" stroke-linejoin="round"/>
      </svg>
      `,
      'image/svg+xml'
    );
  },
  createTextInput(tagInstance) {
    const input = createElement('INPUT', { className: tagInstance.options.classnames.inputElement, tabIndex: 0 });
    input.setAttribute('type', 'text');
    return input;
  },
  createCharsReportPanel(tagInstance) {
    const panel = createElement('DIV', { className: tagInstance.options.classnames.charsCountElement, tabIndex: -1 });
    return panel;
  },
  onTagChange() {},
  initialTags: [],
  maxLength: null
};

export default class TagPanel {
  constructor(element, options = {}) {
    this._createPanel = this._createPanel.bind(this);
    this._createInput = this._createInput.bind(this);
    this._subscribeEvents = this._subscribeEvents.bind(this);
    this._unsubscribeEvents = this._unsubscribeEvents.bind(this);
    this._handleTagInput = this._handleTagInput.bind(this);
    this._handleTagBlur = this._handleTagBlur.bind(this);
    this._handleTagClick = this._handleTagClick.bind(this);
    this._handleTagDelete = this._handleTagDelete.bind(this);
    this._handleGlobalKeyup = this._handleGlobalKeyup.bind(this);
    this._notifyCallbackFn = this._notifyCallbackFn.bind(this);
    this._notifyError = this._notifyError.bind(this);
    this._reportRemaining = this._reportRemaining.bind(this);
    this._initTags = this._initTags.bind(this);
    this.addTag = this.addTag.bind(this);
    this.destroy = this.destroy.bind(this);
    this.getTagList = this.getTagList.bind(this);
    this.removeTag = this.removeTag.bind(this);

    this.options = Object.assign({}, defaultOptions, options);
    this.options.classnames = Object.assign({}, defaultOptions.classnames, options.classnames);

    this.panel = this._createPanel(element, this.options.classnames.tagPanel);
    this.input = this._createInput(this.panel);

    this._subscribeEvents(this);
    this._tagLength = 0;
    this._controlKeyCodes = [13, 8, 27];
    if (this.options.maxLength > 0) {
      this.report = this.options.createCharsReportPanel(this);
      this.panel.parentNode.appendChild(this.report);
      this.charsRemaining = this.options.maxLength;
    }
    this._initTags();
  }

  destroy() {
    this._unsubscribeEvents();
    removeElement(this.panel);
  }

  getTagList() {
    const result = [];
    if (this.panel.hasChildNodes()) {
      const nodes = this.panel.childNodes;
      for (let i = 0; i < nodes.length; i++) {
        const tag = this._getTag(nodes[i], this.options.classnames.tagElement);
        if (tag) {
          result.push(tag);
        }
      }
    }
    return result;
  }

  addTag(text) {
    if (text === null || text === undefined) {
      this._notifyError('Invalid tag');
      return;
    }
    const len = this._tagLength + text.length;
    if (this.options.maxLength && len > this.options.maxLength) {
      this._notifyError('Tag list is already maximun length');
      return;
    }
    this.panel.insertBefore(this._createTagElement(text), this.input);
    setTimeout(() => this._notifyCallbackFn());
  }

  removeTag(tagElement) {
    if (tagElement instanceof HTMLElement) {
      removeElement(tagElement);
      setTimeout(() => this._notifyCallbackFn());
    } else if (typeof tagElement === 'string') {
      const result = [];
      this.panel.childNodes.forEach(el => {
        if (hasText(el, tagElement)) {
          result.push(el);
        }
      });
      result.forEach(el => removeElement(el));
      result = null;
      setTimeout(() => this._notifyCallbackFn());
    }
  }

  _initTags() {
    this.options.initialTags.forEach(tag => this.addTag(tag));
  }

  _notifyCallbackFn() {
    const tagList = this.getTagList();
    this._tagLength = tagList.join('').length;
    this._reportRemaining();
    this.options.onTagChange(tagList);
  }

  _notifyError(message) {
    if (this.options.onError) {
      this.options.onError(message);
    }
  }

  _reportRemaining() {
    if (this.report) {
      this.report.innerHTML = `${this.options.maxLength - this.input.value.length - this._tagLength} remaining`;
    }
  }

  _createTagElement(tag) {
    const tagEl = createElement('DIV', {
      className: this.options.classnames.tagElement,
      tabIndex: this.panel.childNodes.length
    });
    const tagTextEl = createElement('SPAN', { innerHTML: tag });
    const tagIconEl = this.options.createRemoveButton();

    addEvent('click', tagEl, this._handleTagClick);
    addEvent('click', tagIconEl, this._handleTagDelete);
    tagEl.appendChild(tagTextEl);
    tagEl.appendChild(tagIconEl);

    return tagEl;
  }

  _getTag(node, classnames) {
    let text = '';
    let i = 0;
    if (node.nodeType === 1 && node.nodeName === 'DIV' && node.className === classnames && node.hasChildNodes()) {
      while (node.childNodes[i]) {
        if (node.childNodes[i].nodeType === 1 && node.childNodes[i].nodeName === 'SPAN') {
          text = (node.childNodes[i].innerText || '').trim();
          i = -1;
        } else {
          i++;
        }
      }
    }
    return text;
  }

  _createPanel(element, className) {
    const panel = createElement('DIV', { className });
    queryElement(element).appendChild(panel);
    return panel;
  }

  _createInput(panel) {
    const input = this.options.createTextInput(this);
    panel.appendChild(input);
    return input;
  }

  _subscribeEvents() {
    addEvent('keyup', this.input, this._handleTagInput);
    addEvent('blur', this.input, this._handleTagBlur);
    addEvent('keydown', document, this._handleGlobalKeyup);
  }

  _unsubscribeEvents() {
    removeEvent('keyup', this.input, this._handleTagInput);
    removeEvent('blur', this.input, this._handleTagBlur);
    removeEvent('keydown', document, this._handleGlobalKeyup);
  }

  _handleTagBlur() {
    this.beingUpdate = null;
  }

  _handleTagClick(e) {
    e.stopPropagation();
    this.beingUpdate = e.target;
    this.input.value = this.beingUpdate.innerText;
    this.input.focus();
  }

  _handleTagDelete(e) {
    e.stopPropagation();
    this.beingUpdate = null;
    this.removeTag(e.target.parentNode);
  }

  _handleTagInput(e) {
    e.stopPropagation();
    const code = e.keyCode ? e.keyCode : e.which;
    if (
      this.report &&
      this._tagLength + e.target.value.length >= this.options.maxLength &&
      !this._controlKeyCodes.includes(code)
    ) {
      e.preventDefault();
      e.target.value = e.target.value.substring(0, this.options.maxLength - this._tagLength);
      return;
    }
    this._reportRemaining();

    if (code === 13 && e.target.value.length) {
      if (this.beingUpdate) {
        this.beingUpdate.innerText = e.target.value;
        this.beingUpdate = null;
      } else {
        this.addTag(e.target.value.trim());
      }
      e.target.value = '';
      return;
    } else if (code === 8 && !e.target.value.length) {
      const tagEl = this._getTheLastTag(this.panel, this.options.classnames.tagElement);
      if (tagEl) {
        if (this.beingUpdate) {
          this.removeTag(this.beingUpdate.parentNode);
          return;
        }
        tagEl.focus();
        return;
      }
    } else if (code === 27) {
      this.beingUpdate = null;
      e.target.value = '';
    }

    if (this.beingUpdate) {
      this.beingUpdate.innerText = e.target.value;
    }
  }

  _handleGlobalKeyup(e) {
    let code = e.keyCode ? e.keyCode : e.which;
    if (
      code === 8 &&
      document.activeElement.parentNode === this.panel &&
      document.activeElement.nodeType === 1 &&
      document.activeElement.nodeName === 'DIV' &&
      document.activeElement.className === this.options.classnames.tagElement
    ) {
      const focusedEl = document.activeElement;
      if (document.activeElement.previousSibling && document.activeElement.previousSibling.parentNode === this.panel) {
        document.activeElement.previousSibling.focus();
      }
      this.removeTag(focusedEl);
    } else if (code === 27) {
      this.beingUpdate = null;
      this.input.value = '';
    }
  }

  _getTheLastTag(panel, classnames) {
    let i = panel.childNodes.length - 1 || -1;
    let node = null;
    while (((node = panel.childNodes[i]), node)) {
      if (node.nodeType === 1 && node.nodeName === 'DIV' && node.className === classnames) {
        return panel.childNodes[i];
      } else {
        i--;
      }
    }
    return null;
  }
}
